#!/usr/bin/perl
# mg2mus.pl - Version 0.3  May 24, 2003 

# This script converts routes generated by map&guide9 to a format 
# similar to the RAW output of www.mapsonus.com.  The intention of this script
# is entirely to allow the parsing program included in the Empeg application
# gpsapp (which was written specificly for the output of mapsonus) to work for
# directions made by greenflag.  The result is a route with turn waypoints, but
# no additional route shape points.  Until greenflag includes more waypoints, 
# this is all of the waypoint information available.

# To generate a route containing all of the information that is needed by this 
# script, make sure map&guide is set to detais and "coordinates" and "descriptions" 

# Copyright 2003 by Juergen Forscht

# Usage mg2mus.pl <input file>

if ($#ARGV != 0)
{
	print "Usage: $0 <input file>\n";
	exit
}

unless (open(FILE, "<$ARGV[0]"))
{
	print "Could not open $ARGV[0] for reading.";
	exit;
}

open(OUTFILE, ">$ARGV[0].tmp") or die "Can't open output file: $!";

my @lines = <FILE>;
my $turn = 0;
my @route_points;
my $last_desc = "";
my $lon = 0;
my $lat = 0;
my $desc = "";
my $waypoints = -1;
my $last_waypoints = 0;
my $last_part1 = "";
my $last_part2 = "";
my $last_coord = "";


foreach $line (@lines)
{
	# try to match the lines containing maps of the turn points.
	if ($line =~ /^.*?(.*?)(.*?)\'(.*?)\" \/ (.*?)(.*?)\'(.*?)\" \;(.*?);.*?$/)
	{

		# remember last coordinates
		$last_desc = $desc;
		$last_lon = $lon;
		$last_lat = $lat;
			
		# write last turn to outfile
		print OUTFILE "$last_part2$last_part1";

		# put waypoints to waypoint-file
		if ($last_coord ne "") {push(@route_points, "$last_coord")};

		# reset last coordinates
		$last_part2 = "";
		$last_part1 = "";
		$last_coord = "";
		
		#calculate actual coordinates
		$lon = 0.000005+(int(100000*($1+(($2+($3/60))/60))))/100000;
		$lat = 0.000005+(int(100000*($4+(($5+($6/60))/60))))/100000;
		$desc = $7;

		if ($waypoints == -1) 
		{
			print OUTFILE "START $lon $lat {MARK.START} Start Point ($desc)\n";
			$waypoints++;
			next;
		}

		# extract the road-name from description
		if ($7 =~ /^.*?auf (.*?)$/) 
		{	
			$desc = $1;
		};

		#put turns to turn-file 
		if ("$desc" ne "$last_desc")
		{
			$diff_waypoints = $waypoints - $last_waypoints;
			if ($turn > 0) {$last_part2 = "$diff_waypoints 0 0 0 0.0 $last_desc\n"};
			$last_part1 = "TURN $turn $lon $lat $waypoints ";
			$last_waypoints = $waypoints;
			$last_diff = $diff_waypoints;
			$turn++;
		}

		$waypoints++;

		$last_coord = "$lon $lat";
		next;
	}
}

# tbd: print end-point
# if ($last_diff > 1) {$turn--};
$waypoints--;
$diff_waypoints = $waypoints - $last_waypoints;
print OUTFILE "$last_diff 0 0 0 0.0 $last_desc\n";
$turn--;
print OUTFILE "TURN $turn $last_lon $last_lat $waypoints 1 0 0 0 0.0 $last_desc\n";
print OUTFILE "END $lon $lat {MARK.END} End Point ($desc)\n";


print OUTFILE "MIN 0\n".join('  ', @route_points);
close OUTFILE;
